/**
* MLO Library Interface
*/
#pragma once

#ifndef ML_OAPI_H_
#define ML_OAPI_H_

#ifndef APICALL
#ifdef __cplusplus
#   ifdef WIN32
#       ifndef ML_IMPORTS_API
#           define APICALL extern "C" __declspec(dllexport)
#       else
#           define APICALL extern "C" __declspec(dllimport)
#       endif
#   else
#       define APICALL extern "C" 
#   endif // WIN32
#else
#   define APICALL
#endif
#endif


#ifndef MLOSTACKMODE
#   ifdef WIN32
#       define MLOSTACKMODE   __cdecl
#   else
#       define MLOSTACKMODE
#   endif // WIN32
#endif // !MLOSTACKMODE

// Static version of the library
#define MLO_API_VERSION_MAJOR 2
#define MLO_API_VERSION_MINOR 0
#define MLO_API_VERSION_REVISION 2

#define MLADDRLEN       256     // Maximum address length

enum { MLOAPIVERSION = 1 };

// Types
typedef     unsigned char       byte;
typedef     byte                UInt8;
typedef     unsigned short      UInt16;
typedef     unsigned short      USHORT;
typedef     unsigned long       UInt32;
typedef     unsigned long long  UInt64;

typedef		unsigned short      UINT16, *PUINT16;

typedef struct  mloapi mloapi;

/// <summary>
///  Creates an instance of the library
///		
/// </summary>
/// <returns>Pointer to the MLO instance</returns>
APICALL mloapi* MLOSTACKMODE mlo_createInstance();


/// <summary>
///	Opens connection.
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="IP">Address to connect to</param>
/// <returns> Returns 1 in case of success and 0 otherwise</returns>
APICALL int MLOSTACKMODE mlo_Connect(mloapi* instance, char* IP);


/// <summary>
/// Closes connection to instance.
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <returns> Returns 1 in case of success and 0 otherwise</returns>
APICALL int MLOSTACKMODE mlo_Disconnect(mloapi* instance);

/// <summary>
/// Function to read the Board's ID
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <returns>Returns board's ID in case of success and 0 otherwise</returns>
APICALL int MLOSTACKMODE mlo_ReadBoardID(mloapi* instance);

/// <summary>
/// read the Board's serial number
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="SN">reference to Serial number</param>
/// <param name="SerialNumberVersion">reference to Serial number Version</param>
/// <returns>Returns 1 in case of success and 0 otherwise</returns>
APICALL int MLOSTACKMODE mlo_ReadSerialNumber(mloapi* instance, UInt64 * SN, int *SerialNumberVersion);

/// <summary>
/// read the Board's Firmware Revision
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <returns>returns FW revision</returns>
APICALL double MLOSTACKMODE mlo_ReadFirmwareRevision(mloapi* instance);

/// <summary>
/// read the Board's Temperature
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">index for the three temperature sensors (0,1,2)</param>
/// <returns>returns the temperature</returns>
APICALL double MLOSTACKMODE mlo_TempRead(mloapi* instance, int index);

/// <summary>
/// switch the power comming from the "DUT in" port or the "Ref in" port to either go the 4x1 switch and then to the "Thru out" and do the tx test, or to the "VOA out" and do the RX testings.
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">which 2x2 switch (0,1,2,3,4,5,6,7)</param>
/// <param name="data"> 1 for the parrallel and the 0 for the X</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SwitchSelect_2x2(mloapi* instance, unsigned short index, unsigned short data);

/// <summary>
/// switch between the different 4 channels to let it pass to the Thru out port and do the TX testings (single mode)
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">to select the switch but there is one so not useful</param>
/// <param name="data"> 0,1,2,3 to select the path</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SwitchSelect_MEMS_4x1(mloapi* instance, unsigned short index, unsigned short data);

/// <summary>
/// to switch between the different 4 channels to let it pass to the Thru out port and do the TX testings (multimode)
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">to select the switch but there is one so not useful</param>
/// <param name="data">0,1,2,3 to select the path</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SwitchSelect_LBMM_4x1(mloapi* instance, unsigned short index, unsigned short data);

/// <summary>
/// Function to switch between the different 8 channels to let it pass to the Thru out port and do the TX testings (single mode)
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">to select the switch but there is one so not useful</param>
/// <param name="data">0,1,2,3,4,5,6,7 to select the path</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SwitchSelect_MEMS_8x1(mloapi* instance, unsigned short index, unsigned short data);

/// <summary>
/// switch between the different 8 channels to let it pass to the Thru out port and do the TX testings (multimode)
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the switch but there is one so not useful</param>
/// <param name="data">0,1,2,3,4,5,6,7 to select the path</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SwitchSelect_LBMM_8x1(mloapi* instance, unsigned short index, unsigned short data);

/// <summary>
/// read the status of the 2x2 switch
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the switch (0,1,2,3,4,5,6,7)</param>
/// <param name="data">:refrence to the switch state (parallel/X - 1/0)</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadSwitchSelect_2x2(mloapi* instance, unsigned short index, unsigned short *data);


/// <summary>
/// read which channel has been chosen by the user
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the switch but there is one so not useful</param>
/// <param name="data">refrence to the switch state (0,1,2,3)</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadSwitchSelect_MEMS_4x1(mloapi* instance, unsigned short index, unsigned short *data);

/// <summary>
/// read which channel has been chosen by the user
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the switch but there is one so not useful</param>
/// <param name="data">refrence to the switch state (0,1,2,3)</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadSwitchSelect_LBMM_4x1(mloapi* instance, unsigned short index, unsigned short *data);

/// <summary>
/// read which channel has been chosen by the user
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the switch but there is one so not useful</param>
/// <param name="data">refrence to the switch state (0,1,2,3,4,5,6,7)</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadSwitchSelect_MEMS_8x1(mloapi* instance, unsigned short index, unsigned short *data);

/// <summary>
/// read which channel has been chosen by the user
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the switch but there is one so not useful</param>
/// <param name="data">refrence to the switch state (0,1,2,3,4,5,6,7)</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadSwitchSelect_LBMM_8x1(mloapi* instance, unsigned short index, unsigned short *data);

/// <summary>
/// read power with calibration
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the PM (0,1,2,3,4,5,6,7)</param>
/// <param name="data">refrence to the power value (calibrated)</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadOpticalPower(mloapi* instance, unsigned short index, double* data);

/// <summary>
/// write the attenuation value with calibration
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">to select the PM (0,1,2,3,4,5,6,7)</param>
/// <param name="data">attenuation value with calibration</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SetVOAAttenuation(mloapi* instance, unsigned short index, double data);

/// <summary>
/// read the attenuation value with calibration
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the PM (0,1,2,3,4,5,6,7)</param>
/// <param name="data">reference to the attenuation value with calibration</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadVOAAttenuation(mloapi* instance, unsigned short index, double* data);

/// <summary>
/// read the Box type
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="Type">reference to the Box type
/// 0 = LR,
/// 1 = SR,
/// 2 = CWDM4,
/// 3 = LR4,
/// 4 = SR4,
/// 5 = PSM4,
/// 6 = SR8,
/// 7 = LR8</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadTypeFromEEPROM(mloapi* instance, unsigned short* Type);

/// <summary>
/// enable/disable the screen for a touch control on the front panel of the MLOx`
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">not used default 0 </param>
/// <param name="enable">true for enable false otherwise</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ScreenControl(mloapi* instance, unsigned short index, bool enable);

/// <summary>
/// read the VOA insertion loss value
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the PM (0,1,2,3,4,5,6,7)</param>
/// <param name="Loss">reference to the loss insertion values</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_ReadVOAInsertionLoss(mloapi* instance, unsigned short index, double* Loss);

/// <summary>
/// write the Voa insertion loss value
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the PM (0,1,2,3,4,5,6,7)</param>
/// <param name="Loss">loss insertion value</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_WriteVOAInsertionLoss(mloapi* instance, unsigned short index, double Loss);

/// <summary>
/// read the 4x1 switch insertion loss value
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the PM (0,1,2,3)</param>
/// <param name="Loss">reference to the loss insertion values</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_Read4x1SwitchInsertionLoss(mloapi* instance, unsigned short index, double* Loss);

/// <summary>
/// write the 4x1switch insertion loss value
/// </summary>
/// <param name="instance">Pointer to instance.</param>
/// <param name="index">select the PM (0,1,2,3)</param>
/// <param name="Loss">loss insertion value</param>
/// <returns>True for success and false otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_Write4x1SwitchInsertionLoss(mloapi* instance, unsigned short index, double Loss);

#pragma region MLRESERVED

/// <summary>
/// Set wavelength.
/// </summary>
/// <param name="instance">API instance pointer</param>
/// <param name="wavelength">Wavelength value (in nm).</param>
/// <returns>True if wavelength is calibrated. False otherwise.</returns>
APICALL bool MLOSTACKMODE mlo_SetWavelength(mloapi* instance, unsigned short wavelength);

/// <summary>
/// Add a custom wavelength.
/// </summary>
/// <param name="instance">API instance pointer</param>
/// <param name="wavelength">Wavelength value (in nm).</param>
/// <returns>True if wavelength is added. False otherwise.</returns>
/// <remarks>
/// If the wavelength is previously assigned, the call updates the internal index for calibration.
/// </remarks>
APICALL bool MLOSTACKMODE mlo_AddWavelength(mloapi* instance, unsigned short wavelength);

/// <summary>
/// Reset to type's default wavelength.
/// </summary>
/// <param name="instance">API instance pointer</param>
APICALL bool MLOSTACKMODE mlo_SetDefaultWavelength(mloapi* instance);

APICALL bool MLOSTACKMODE mlo_ReadVOACalibration(mloapi* instance, unsigned short index, double* VOACalib);

APICALL bool MLOSTACKMODE mlo_WriteVOACalibration(mloapi* instance, unsigned short index, double* VOACalib);

APICALL bool MLOSTACKMODE mlo_ReadVOADefinedValues(mloapi* instance, unsigned short index, UINT16* VOADefinedValues);

APICALL bool MLOSTACKMODE mlo_WriteVOADefinedValues(mloapi* instance, unsigned short index, UINT16* VOADefinedValues);

APICALL bool MLOSTACKMODE mlo_ReadTAPCalibration(mloapi* instance, unsigned short index, double* TAPCalib);

APICALL bool MLOSTACKMODE mlo_WriteTAPCalibration(mloapi* instance, unsigned short index, double TAPCalib);

APICALL bool MLOSTACKMODE mlo_ReadOpticalPowerBits(mloapi* instance, unsigned short index, unsigned short* data);

APICALL bool MLOSTACKMODE mlo_SetVOAAttenuationBits(mloapi* instance, unsigned short index, unsigned short data);

APICALL bool MLOSTACKMODE mlo_ReadVOAAttenuationBits(mloapi* instance, unsigned short index, unsigned short* data);

APICALL bool MLOSTACKMODE mlo_WriteTypeToEEPROM(mloapi* instance, unsigned short Type);

#pragma endregion

#endif